@@ -1,6 +1,5 @@
 package XAO::Web;
 use strict;
-use CGI;
 use Error qw(:try);
 use XAO::Utils;
 use XAO::Projects;
@@ -14,7 +13,7 @@ use XAO::Errors qw(XAO::Web);
 # XAO::Web version number. Hand changed with every release!
 #
 use vars qw($VERSION);
-$VERSION='1.07';
+$VERSION='1.45';
 
 ###############################################################################
 
@@ -38,8 +37,9 @@ XAO::Web - XAO Web Developer, dynamic content building suite
 =head1 DESCRIPTION
 
 Please read L<XAO::Web::Intro> for general overview and setup
-instructions. Check also misc/samplesite for code examples and a generic
-site setup.
+instructions, and please read L<XAO::DO::Web::Page> for an overview
+of the templating system. Check also misc/samplesite for code examples
+and a generic site setup.
 
 XAO::Web module provides a frameworks for loading site configuration and
 executing objects and templates in the site context. It is used in
@@ -131,7 +131,7 @@ other than 'xaoweb'. This is used by Apache::XAO to get 'maptodir' and
 'external' mappings. Default is to look for xaoweb only records.
 
 =cut
- 
+
 sub analyze ($$;$$) {
     my ($self,$patharr,$sitename,$allow_other_types)=@_;
 
@@ -274,7 +274,7 @@ Returns site configuration object reference.
 
 sub config ($) {
     my $self=shift;
-    return $self->{siteconfig} ||
+    return $self->{'siteconfig'} ||
         throw XAO::E::Web "config - no configuration object";
 }
 
@@ -285,10 +285,11 @@ sub config ($) {
 Executes given `path' using given `cgi' environment. Prints results to
 standard output and uses CGI object methods to send header.
 
-B<Note:> Execute() is not re-entry safe currently! Meaning that if you
-create a XAO::Web object in any method called inside of execute() loop
-and then call execute() on that newly created XAO::Web object the system
-will fail and no useful results will be produced.
+B<Note:> Execute() changes global projects context and is not re-entry safe
+currently! Meaning that if you create a XAO::Web object in any method
+called inside of execute() loop and then call execute() on that newly
+created XAO::Web object the system will fail and no useful results will
+be produced.
 
 =cut
 
@@ -296,54 +297,104 @@ sub execute ($%) {
     my $self=shift;
     my $args=get_args(\@_);
 
-    ##
+    # Setting dprint/eprint to Apache methods if needed
+    #
+    my $old_logprint_handler;
+    if($args->{'apache'}) {
+        $old_logprint_handler=XAO::Utils::set_logprint_handler(sub {
+            $args->{'apache'}->server->warn($_[0]);
+        });
+    }
+
+    # Setting the current project context to our site.
+    #
+    $self->set_current();
+
     # We check if the site has a mapping for '/internal-error' in
-    # path_mapping_table. If it has we wrap expand() into the try block
+    # path_mapping_table. If it has we wrap process() into the try block
     # and execute /internal-error if we get an error.
     #
-    my ($pagetext,$header);
+    my $pagetext;
     try {
-        ($pagetext,$header)=$self->expand($args);
+        $pagetext=$self->process($args);
     }
     otherwise {
         my $e=shift;
-        my $path="/internal-error/index.html";
+
+        # Under mod_perl we get apache's internal exceptions for genuine apache
+        # problems (timeouts, etc). These are not re-throwable apparently,
+        # so we wrap them into Error::Simple.
+        #
+        if($e->isa('APR::Error')) {
+            $e=Error::Simple->new("$e");
+        }
+
+        $self->config->header_args(
+            -Status         => '500 Internal Error',
+            -expires        => 'now',
+            -cache_control  => 'no-cache',
+        );
+
+        my $edata=$self->clipboard->get('/internal_error') || { };
+
+        my $path=$edata->{'display_path'} || '/internal-error/index.html';
         my $pd=$self->analyze($path);
+
         if($pd && $pd->{'type'} eq 'xaoweb' && $pd->{'objname'} ne 'Default') {
             eprint "$e";
-            $self->clipboard->put("internal_error" => {
-                error       => $e,
-                path        => $args->{path},
-                pagedesc    => $self->clipboard->get('pagedesc'),
-            });
-            ($pagetext,$header)=$self->expand($args,{
+
+            $edata->{'message'}||="$e";
+            $edata->{'code'}||='UNKNOWN';
+            $edata->{'path'}||=$args->{'path'};
+            $edata->{'pagedesc'}||=$self->clipboard->get('pagedesc');
+
+            $self->clipboard->put(internal_error => $edata);
+
+            $pagetext=$self->process($args,{
                 path        => $path,
+                template    => undef,
                 pagedesc    => $pd,
             });
         }
         else {
+            XAO::Utils::set_logprint_handler($old_logprint_handler) if $old_logprint_handler;
             throw $e;
         }
     };
 
-    ##
+    # We need to call "header" for CGI to do its magic on it. We
+    # typically will get an empty string in mod_perl environment, and the
+    # header will be sent to Apache by CGI.
+    #
+    my $header=$self->config->header;
+
     # If we get the header then it was not printed before and we are
     # expected to print out the page. This is almost always true except
     # when page includes something like Redirect object.
     #
-    if(defined($header)) {
-        if(my $r=$args->{apache}) {
+    if(defined $header) {
+        if(my $r=$args->{'apache'}) {
             my $h=$self->config->header_args;
-            while(my ($n,$v)=each %$h) {
-                $r->header_out($n => $v);
-                $r->err_header_out($n => $v);
-            }
+
             if($mod_perl::VERSION && $mod_perl::VERSION >= 1.99) {
+                # This is accomplished by CGI when config->header is
+                # called above, and it does not work properly anyway
+                #
+                ### while(my ($n,$v)=each %$h) {
+                ###     dprint "n='$n' v='$v'";
+                ###     $r->headers_out->set($n => $v);
+                ###     $r->err_headers_out->set($n => $v);
+                ### }
                 $r->content_type('text/html') unless $r->content_type;
             }
             else {
+                while(my ($n,$v)=each %$h) {
+                    $r->header_out($n => $v);
+                    $r->err_header_out($n => $v);
+                }
                 $r->send_http_header;
             }
+
             $r->print($pagetext) unless $r->header_only;
         }
         else {
@@ -351,6 +402,14 @@ sub execute ($%) {
                   $pagetext;
         }
     }
+
+    # Cleaning up site configuration
+    #
+    $self->config->cleanup(mode => 'after');
+
+    # Restoring the default dprint/eprint handling
+    #
+    XAO::Utils::set_logprint_handler($old_logprint_handler) if $old_logprint_handler;
 }
 
 ###############################################################################
@@ -362,19 +421,22 @@ just the text of the page in scalar context and page content plus header
 content in array context.
 
 This is normally used in scripts to execute only a particular template
-and get results of execution.
+and to get results of execution. BUT this code is also used as part of
+the normal execute().
 
 `Objargs' argument may refer to a hash of additional parameters to be
 passed to the template being executed.
 
 Example:
 
- my $report=$web->expand(cgi     => CGI->new,
-                         path    => '/bits/stat-report',
-                         objargs => {
-                             CUSTOMER_ID => '123X234Z',
-                             MIN_TIME    => time - 86400 * 7,
-                         });
+ my $report=$web->expand(
+     cgi     => XAO::Objects->new(objname => 'CGI'),
+     path    => '/bits/stat-report',
+     objargs => {
+         CUSTOMER_ID => '123X234Z',
+         MIN_TIME    => time - 86400 * 7,
+     },
+ );
 
 See also lower level process() method.
 
@@ -384,43 +446,62 @@ sub expand ($%) {
     my $self=shift;
     my $args=get_args(\@_);
 
-    ##
+    $self->set_current;
+
     # Processing the page and getting its text. Setting dprint and
     # eprint to use Apache logging if there is a reference to Apache
     # request given to us.
     #
-    my $pagetext;
-    if($args->{apache}) {
-        my $old_logprint_handler=XAO::Utils::set_logprint_handler(sub {
-            $args->{apache}->server->warn($_[0]);
-        });
+    my $pagetext=$self->process($args);
 
-        $pagetext=$self->process($args);
-
-        XAO::Utils::set_logprint_handler($old_logprint_handler);
-    }
-    else {
-        $pagetext=$self->process($args);
-    }
-
-    ##
     # In scalar context (normal cases) we return only the resulting page
     # text. In array context (compatibility) we return header as well.
     #
-    my $siteconfig=$self->config;
     if(wantarray) {
-        my $header=$siteconfig->header;
-        $siteconfig->cleanup;
+        eprint "Calling ".ref($self)."::expand in ARRAY context is obsolete";
+        my $header=$self->config->header;
+        $self->config->cleanup(mode => 'after');
         return ($pagetext,$header);
     }
     else {
-        $siteconfig->cleanup;
+        $self->config->cleanup(mode => 'after');
         return $pagetext;
     }
 }
 
 ###############################################################################
 
+sub _expand_list ($$) {
+    my ($self,$autolist)=@_;
+
+    my $content='';
+
+    if(!$autolist) {
+        return '';
+    }
+    elsif(ref($autolist) eq 'ARRAY') {
+        for(my $i=0; $i<@$autolist; $i+=2) {
+            my ($objname,$objargs)=@{$autolist}[$i,$i+1];
+            my $obj=XAO::Objects->new(objname => $objname);
+            $content.=$obj->expand($objargs);
+        }
+    }
+    elsif(ref($autolist) eq 'HASH') {
+        foreach my $objname (keys %{$autolist}) {
+            my $obj=XAO::Objects->new(objname => $objname);
+            $content.=$obj->expand($autolist->{$objname});
+        }
+    }
+    else {
+        throw XAO::E::Web "process - don't know how to handle ($autolist)," .
+                          " must be a hash or an array reference";
+    }
+
+    return $content;
+}
+
+###############################################################################
+
 =item process (%)
 
 Takes the same arguments as the expand() method returning expanded page
@@ -437,25 +518,17 @@ sub process ($%) {
     my $siteconfig=$self->config;
     my $sitename=$self->sitename;
 
-    ##
     # Making sure path starts from a slash
     #
-    my $path=$args->{path} || throw XAO::E::Web "expand - no 'path' given";
+    my $path=$args->{'path'} || throw XAO::E::Web "process - no 'path' given";
     $path='/' . $path;
     $path=~s/\/{2,}/\//g;
 
-    ##
-    # Setting the current project context to our site.
-    #
-    $self->set_current();
-
-    ##
     # Resetting page text stack in case it was terminated abnormally
     # before and we're in the same process/memory.
     #
     XAO::PageSupport::reset();
 
-    ##
     # Analyzing the path. We have to do that up here because the object
     # might specify that we should not touch CGI.
     #
@@ -467,7 +540,6 @@ sub process ($%) {
         $pd=$self->analyze(\@path);
     }
 
-    ##
     # Figuring out current active URL. It might be the same as base_url
     # and in most cases it is, but it just as well might be different.
     #
@@ -477,10 +549,10 @@ sub process ($%) {
     # CGI usage.
     #
     my $active_url;
-    my $apache=$args->{apache};
-    my $cgi=$args->{cgi};
+    my $apache=$args->{'apache'};
+    my $cgi=$args->{'cgi'};
     if(!$cgi) {
-        $cgi=$pd->{no_cgi} ? CGI->new('foo=bar') : CGI->new;
+        $cgi=XAO::Objects->new(objname => 'CGI', no_cgi => $pd->{'no_cgi'});
     }
     if($apache) {
         $active_url="http://" . $apache->hostname;
@@ -506,7 +578,6 @@ sub process ($%) {
             # dprint "<2.8 $active_url";
         }
 
-        ##
         # Trying to understand if rewrite module was used or not. If not
         # - adding sitename to the end of guessed URL.
         #
@@ -515,13 +586,11 @@ sub process ($%) {
         }
     }
 
-    ##
     # Eating extra slashes
     #
     chop($active_url) while $active_url =~ /\/$/;
     $active_url=~s/(?<!:)\/\//\//g;
 
-    ##
     # Figuring out secure URL
     #
     my $active_url_secure;
@@ -547,10 +616,10 @@ sub process ($%) {
     # Checking if we have base_url, assuming active_url if not.
     # Ensuring that URL does not end with '/'.
     #
-    if($siteconfig->defined("base_url")) {
+    if($siteconfig->defined('base_url')) {
         my $url=$siteconfig->get('base_url');
         $url=~/^http:/i ||
-            throw XAO::E::Web "expand - bad base_url ($url) for sitename=$sitename";
+            throw XAO::E::Web "process - bad base_url ($url) for sitename=$sitename";
         my $nu=$url;
         chop($nu) while $nu =~ /\/$/;
         $siteconfig->put(base_url => $nu) if $nu ne $url;
@@ -569,15 +638,28 @@ sub process ($%) {
         $siteconfig->put(base_url_secure => $active_url_secure);
         dprint "No base_url for sitename '$sitename'; assuming base_url=$active_url, base_url_secure=$active_url_secure";
     }
-  
-    ##
+
     # Checking if we're running under mod_perl
     #
-    my $mod_perl=($apache || $ENV{MOD_PERL}) ? 1 : 0;
+    my $mod_perl=($apache || $ENV{'MOD_PERL'}) ? 1 : 0;
     $siteconfig->clipboard->put(mod_perl => $mod_perl);
     $siteconfig->clipboard->put(mod_perl_request => $apache);
 
-    ##
+    # Checking if a charset is known for the site. If it is, setting
+    # it up for CGI-params decoding and for output.
+    #
+    if(my $charset=$siteconfig->get('charset')) {
+        if($cgi->can('set_param_charset')) {
+            $cgi->set_param_charset($charset);
+        }
+        else {
+            eprint "CGI object we have does not support set_param_charset";
+        }
+        $siteconfig->header_args(
+            -Charset   => $charset,
+        );
+    }
+
     # Putting CGI object into site configuration. The special case is
     # 'no_cgi' in the path_mapping_table which means that the object is
     # going to handle CGI arguments itself. It can be useful if it needs
@@ -588,27 +670,39 @@ sub process ($%) {
     $siteconfig->embedded('web')->disable_special_access;
 
     ##
-    # Checking for directory index url without trailing slash and
-    # redirecting with appended slash if this is the case.
+    # Traditionally URLs that do not end with .foo are considered
+    # directories and get an internal redirect to path/index.html
+    # Sometimes it is desirable to be able to pass down any URLs without
+    # a forced redirect -- this is controlled by 'urlstyle' parameter
+    # set to 'raw'.
     #
-    if($pd->{patharr}->[-1] !~ /\.\w+$/) {
-        my $pd=$self->analyze([ @{$pd->{patharr}},'index.html' ]);
-        #use Data::Dumper; dprint "pd=",Dumper($pd);
-        if($pd->{objname} ne 'Default') {
-            my $newpath=$siteconfig->get('base_url') . $path . '/';
-            dprint "Redirecting $path to $newpath";
-            $siteconfig->header_args(
-                -Location   => $newpath,
-                -Status     => 301,
-            );
-            return "Directory index redirection\n";
+    my $urlstyle=$pd->{'urlstyle'} || 'files';
+    if($urlstyle eq 'files') {
+        if($pd->{'patharr'}->[-1] !~ /\.\w+$/) {
+            my $pd=$self->analyze([ @{$pd->{'patharr'}},'index.html' ]);
+            #use Data::Dumper; dprint "pd=",Dumper($pd);
+            if($pd->{'objname'} ne 'Default') {
+                my $newpath=$siteconfig->get('base_url') . $path . '/';
+                dprint "Redirecting $path to $newpath";
+                $siteconfig->header_args(
+                    -Location   => $newpath,
+                    -Status     => 301,
+                );
+                return "Directory index redirection\n";
+            }
         }
     }
+    elsif($urlstyle eq 'raw') {
+        # nothing
+    }
+    else {
+        eprint "Unknown urlstyle '$urlstyle' for $path";
+    }
 
     ##
     # Separator for the error_log :)
     #
-    if(XAO::Utils::get_debug()) {
+    if(XAO::Utils::get_debug() && !$args->{'quieter'}) {
         my @d=localtime;
         my $date=sprintf("%02u:%02u:%02u %u/%02u/%04u",$d[2],$d[1],$d[0],$d[4]+1,$d[3],$d[5]+1900);
         undef(@d);
@@ -621,12 +715,10 @@ sub process ($%) {
     #
     $siteconfig->clipboard->put(pagedesc => $pd);
 
-    ##
     # We accumulate page content here
     #
     my $pagetext='';
 
-    ##
     # Setting expiration time in the page header to immediate
     # expiration. If that's not what the page wants -- it can override
     # these.
@@ -636,50 +728,32 @@ sub process ($%) {
         -cache_control  => 'no-cache',
     );
 
-    ##
     # Do we need to run any objects before executing? A good place to
     # turn on debug mode if required using Debug object.
     #
-    my $autolist=$siteconfig->get('auto_before');
-    if($autolist) {
-        if(ref($autolist) eq 'ARRAY') {
-            for(my $i=0; $i<@$autolist; $i+=2) {
-                my ($objname,$objargs)=@{$autolist}[$i,$i+1];
-                my $obj=XAO::Objects->new(objname => $objname);
-                $pagetext.=$obj->expand($objargs);
-            }
-        }
-        elsif(ref($autolist) eq 'HASH') {
-            foreach my $objname (keys %{$autolist}) {
-                my $obj=XAO::Objects->new(objname => $objname);
-                $pagetext.=$obj->expand($autolist->{$objname});
-            }
-        }
-        else {
-            throw XAO::E::Web "process - don't know how to handle auto_before ($autolist)," .
-                              " must be a hash or an array reference";
-        }
-    }
+    $pagetext.=$self->_expand_list($siteconfig->get('auto_before'));
 
-    ##
     # Preparing object arguments out of standard ones, object specific
     # once from template paths and supplied hash (in that order of
     # preference).
     #
     my $objargs={
-        path => $pd->{path},
-        fullpath => $pd->{fullpath},
-        prefix => $pd->{prefix},
+        path        => $pd->{'path'},
+        fullpath    => $pd->{'fullpath'},
+        prefix      => $pd->{'prefix'},
     };
-    $objargs=merge_refs($objargs,$pd->{objargs},$args->{objargs});
+    $objargs=merge_refs($objargs,$pd->{'objargs'},$args->{'objargs'});
 
-    ##
     # Loading page displaying object and executing it.
     #
-    my $obj=XAO::Objects->new(objname => 'Web::' . $pd->{objname});
+    my $obj=XAO::Objects->new(objname => 'Web::' . $pd->{'objname'});
     $pagetext.=$obj->expand($objargs);
 
-    ##
+    # Do we need to run any objects after executing? A good place to
+    # dump benchmark statistics for example.
+    #
+    $pagetext.=$self->_expand_list($siteconfig->get('auto_after'));
+
     # Done!
     #
     return $pagetext;
@@ -705,7 +779,7 @@ sub new ($%) {
     ##
     # Getting site name
     #
-    my $sitename=$args->{sitename} ||
+    my $sitename=$args->{'sitename'} ||
         throw XAO::E::Web "new - required parameter missing (sitename)";
 
     ##
@@ -716,8 +790,10 @@ sub new ($%) {
         ##
         # Creating configuration.
         #
-        $siteconfig=XAO::Objects->new(sitename => $sitename,
-                                      objname => 'Config');
+        $siteconfig=XAO::Objects->new(
+            sitename => $sitename,
+            objname => 'Config',
+        );
 
         ##
         # Always embedding at least web config and a hash
@@ -729,33 +805,44 @@ sub new ($%) {
         # Running initialization, this is where parameters are inserted and
         # normally FS::Config gets embedded.
         #
-        $siteconfig->init();
+        $siteconfig->init($args->{'init_args'} || ());
 
         ##
         # Creating an entry in in-memory projects repository
         #
-        XAO::Projects::create_project(name => $sitename,
-                                      object => $siteconfig,
-                                     );
+        XAO::Projects::create_project(
+            name        => $sitename,
+            object      => $siteconfig,
+        );
     }
 
-    ##
-    # Cleaning up the configuration. Useful even if it was just created
-    # as it will unlock tables in the database for instance.
+    # CGI in args is not supported any more, needs to be passed in execute
     #
-    $siteconfig->cleanup;
+    $args->{'cgi'} &&
+        throw XAO::E::Web "new - 'cgi' argument to 'new' is not supported, pass it to 'execute'";
 
-    ##
-    # If we are given a CGI reference then putting it into the
-    # configuration.
+    # This helps Mailer to be called outside of web context.
+    # TODO: Probably need some better initialization strategy, this does
+    # not feel as the Right Thing
     #
-    if($args->{cgi}) {
-        $siteconfig->embedded('web')->enable_special_access;
-        $siteconfig->cgi($args->{cgi});
-        $siteconfig->embedded('web')->disable_special_access;
+    my $url=$siteconfig->get('base_url');
+    if($url) {
+        $url=~/^http:/i ||
+            throw XAO::E::Web "new - bad base_url ($url) for sitename=$sitename";
+        my $nu=$url;
+        chop($nu) while $nu =~ /\/$/;
+        $siteconfig->put(base_url => $nu) if $nu ne $url;
+
+        $url=$siteconfig->get('base_url_secure');
+        if(!$url) {
+            $url=$siteconfig->get('base_url');
+            $url=~s/^http:/https:/i;
+        }
+        $nu=$url;
+        chop($nu) while $nu =~ /\/$/;
+        $siteconfig->put(base_url_secure => $nu);
     }
 
-    ##
     # Done
     #
     bless {
@@ -766,6 +853,28 @@ sub new ($%) {
 
 ###############################################################################
 
+sub check_uri_access ($$) {
+    my ($self,$uri)=@_;
+
+    # By convention we disallow access to /bits/ and /CVS/ for security
+    # reasons. If needed the site can override these or add other
+    # regex'es into path_deny_table
+    #
+    my $pdtc=$self->config->get('path_deny_table_compiled');
+    if(!$pdtc) {
+        my $pdt=merge_refs({
+            '/bits/'        => 1,
+            '/CVS/'         => 1,
+        },$self->config->get('path_deny_table') || { });
+        $pdtc=[ map { qr/$_/ } grep { $pdt->{$_} } keys %$pdt ];
+        $self->config->put('path_deny_table_compiled' => $pdtc);
+    }
+
+    return ! grep { $uri =~ $_ } @$pdtc;
+}
+
+###############################################################################
+
 =item set_current ()
 
 Sets the current site as the current project in the sense of XAO::Projects.
@@ -774,7 +883,15 @@ Sets the current site as the current project in the sense of XAO::Projects.
 
 sub set_current ($) {
     my $self=shift;
+
     XAO::Projects::set_current_project($self->sitename);
+
+    # Cleaning up the configuration. Useful even if it was just created
+    # as it will unlock tables in the database for instance.
+    # We call it here because cleanup code may rely on the project being
+    # active.
+    #
+    $self->config->cleanup(mode => 'before');
 }
 
 ###############################################################################
@@ -787,7 +904,7 @@ Returns site name.
 
 sub sitename ($) {
     my $self=shift;
-    $self->{sitename} || throw XAO::E::Web "sitename - no site name";
+    $self->{'sitename'} || throw XAO::E::Web "sitename - no site name";
 }
 
 ###############################################################################
@@ -813,4 +930,6 @@ Copyright (c) 2001-2004 Andrew Maltsev, XAO Inc.
 Recommended reading:
 L<XAO::Objects>,
 L<XAO::Projects>,
-L<XAO::DO::Config>.
+L<XAO::DO::Config>,
+L<XAO::Web::Intro>,
+L<XAO::DO::Web::Page>.
