@@ -2,7 +2,7 @@ package testcases::PageSupport;
 use strict;
 use Data::Dumper;
 use XAO::Utils;
-use base qw(testcases::base);
+use base qw(XAO::testcases::Web::base);
 
 use XAO::PageSupport;
 
@@ -202,7 +202,7 @@ sub test_parse {
                             a.pa_ss
                             default.path="/bits/header-normal"
                             default.pass
-                          %><!--//javascript--><%End%>Something) => [
+                          %><!--//javascript-->foo<!--bar-->baz<!--[if IE]>CODE<![endif]--><!--[if (gt IE 9)|!(IE)]><!--><html lang="en"><!--<![endif]--><%End%>Something) => [
             {   objname => 'Condition',
                 args    => {
                     'a.cgiparam'    => [
@@ -227,8 +227,8 @@ sub test_parse {
                     ],
                 },
             },
-            {   text    => '<!--//javascript-->',
-            },
+            {   text    => '<!--//javascript-->foo' },
+            {   text    => 'baz<!--[if IE]>CODE<![endif]--><!--[if (gt IE 9)|!(IE)]><!--><html lang="en"><!--<![endif]-->' },
         ],
         # Sample from the man page
         q(Text <%Object a=A b="B" c={<%C/f ca={CA}%>} d='D' e={'<$E$>'}%>) => [
@@ -274,9 +274,82 @@ sub test_parse {
     }
 }
 
+###############################################################################
+
+sub test_peek {
+    my $self=shift;
+
+    my $text1="t1:{abcdefg}";
+    my $text2="t2:{ABCDEFGH}";
+    my $text3="t3:{123456789}";
+
+    my $tpsub=sub ($) {
+        my $name=shift;
+        XAO::PageSupport::push();
+        XAO::PageSupport::addtext('foo');
+        XAO::PageSupport::addtext('bar');
+        XAO::PageSupport::push();
+        XAO::PageSupport::addtext('baz');
+        my $got=XAO::PageSupport::pop();
+        $self->assert($got eq 'baz',
+            "Expected 'baz', got '$got' ($name-1)");
+        $got=XAO::PageSupport::pop();
+        $self->assert($got eq 'foobar',
+            "Expected 'foobar', got '$got' ($name-2)");
+    };
+
+    $tpsub->('t0');
+
+    my $bm1=XAO::PageSupport::bookmark();
+    ### dprint "..bookmark1=$bm1";
+
+    XAO::PageSupport::addtext($text1);
+
+    my $got1=XAO::PageSupport::peek($bm1);
+
+    $self->assert($got1 eq $text1,
+        "Expected '$text1', got '$got1' (t1)");
+
+    $tpsub->('t2');
+
+    $got1=XAO::PageSupport::peek($bm1);
+
+    $self->assert($got1 eq $text1,
+        "Expected '$text1', got '$got1' (t3)");
+
+    my $bm2=XAO::PageSupport::bookmark();
+    ### dprint "..bookmark2=$bm2";
+
+    XAO::PageSupport::addtext($text2);
+
+    my $got2=XAO::PageSupport::peek($bm2);
+
+    $self->assert($got2 eq $text2,
+        "Expected '$text2', got '$got2' (t4)");
+
+    $got1=XAO::PageSupport::peek($bm1);
+
+    $self->assert($got1 eq $text1.$text2,
+        "Expected '$text1$text2', got '$got1' (t5)");
+
+    $tpsub->('t6');
+
+    $got2=XAO::PageSupport::peek($bm2);
+
+    $self->assert($got2 eq $text2,
+        "Expected '$text2', got '$got2' (t7)");
+
+    $got1=XAO::PageSupport::peek($bm1);
+
+    $self->assert($got1 eq $text1.$text2,
+        "Expected '$text1$text2', got '$got1' (t8)");
+}
+ 
 sub test_textadd {
     my $self=shift;
 
+    XAO::PageSupport::reset();
+
     XAO::PageSupport::addtext("123abcABC");
     XAO::PageSupport::push();
     XAO::PageSupport::addtext("INNER");
@@ -285,10 +358,10 @@ sub test_textadd {
     my $outer=XAO::PageSupport::pop();
 
     $self->assert($inner eq 'INNER',
-                  "Inner block is not correct");
+                  "Inner block is not correct (expected 'INNER', got '$inner')");
 
     $self->assert($outer eq '123abcABC',
-                  "Outer block is not correct");
+                  "Outer block is not correct (expected '123abcABC', got '$outer'");
 
     $inner=$outer='';
     for(1..10) {
