@@ -2,7 +2,7 @@ package testcases::WebUtility;
 use strict;
 use XAO::Projects;
 
-use base qw(testcases::base);
+use base qw(XAO::testcases::Web::base);
 
 sub test_number_ordinal_suffix {
     my $self=shift;
